<?php
namespace Ep\Woo\Plugin\Product;

use Ep\Woo\Product\ProductInterface;
use Ak\Wp\Plugin\PluginInterface;
use Psr\Cache\CacheItemPoolInterface;
use Zend\View\Renderer\RendererInterface;

/**
 * Dodaje proizvode u ux_slider
 *
 * @package Ep\Woo\Plugin\Product
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link http://gitlab.ep.local/ep-framework/woocommerce
 */
final class ProductCarousel implements PluginInterface
{
    /** @var RendererInterface */
    private $renderer;
    /** @var CacheItemPoolInterface */
    private $cache;

    /**
     * Konstruktor.
     *
     * @param RendererInterface $renderer
     * @param CacheItemPoolInterface $cache
     */
    public function __construct(RendererInterface $renderer, CacheItemPoolInterface $cache)
    {
        $this->renderer = $renderer;
        $this->cache = $cache;
    }

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function attach(): void
    {
        add_action('init', [$this, 'init']);
    }

    /**
     * Inicijalizacija.
     */
    public function init(): void
    {
        add_shortcode('product_carousel', [$this, 'render']);
    }

    /**
     * Renda carousel.
     *
     * @param array|string $atts
     * @param $content
     *
     * @return string
     *
     * @throws \Psr\Cache\InvalidArgumentException
     */
    public function render(/** @noinspection PhpUnusedParameterInspection */ $atts, $content): string
    {
        $skus = array_values(array_filter(array_map('trim', explode("\n", strip_tags($content)))));
        asort($skus);
        $skus = array_values($skus);

        $cacheKey = 'product_carousel.' . md5(implode(';', $skus));
        $cacheItem = $this->cache->getItem($cacheKey);

        if (!$cacheItem->isHit()) {
            /** @var \Ep\Gath\Plugin\Products $gathProducts */
            $gathProducts = apply_filters('ep_gath_products', null);
            $posts = $gathProducts->getProductsBySkus($skus);
            $product_ids = array_map(function(\WP_Post $post) { return $post->ID; }, $posts);
            $products = [];

            foreach ($product_ids as $product_id) {
                /** @var \WC_Product[]|ProductInterface[] $products */
                $products[] = wc_get_product($product_id);
            }

            $cols = [];

            foreach ($products as $product) {
                $cols[] = sprintf('[col_inner span="4" span__sm="12"]%s[/col_inner]', $this->renderProduct($product));
            }

            $rows = [];

            foreach (array_chunk($cols, 3) as $chunk) {
                $rows[] = sprintf('[row_inner style="small" width="custom" v_align="equal" h_align="center"]%s[/row_inner]', implode("\n", $chunk));
            }

            $slider = sprintf('[ux_slider style="container" arrows="false" bullets="false"]%s[/ux_slider]', implode("\n", $rows));

            $rendered = sprintf('<div class="ProductCarousel">%s</div>', do_shortcode($slider));

            $cacheItem->set($rendered);
            $this->cache->save($cacheItem);
        }

        return $cacheItem->get();
    }

    /**
     * @param ProductInterface|\WC_Product $product
     * @return string
     */
    private function renderProduct($product)
    {
        return $this->renderer->render('@woo/product-carousel-item.html.twig', [
            'product' => $product,
            'product_url' => get_permalink($product->get_id()),
            'image' => $product->get_image('woocommerce_thumbnail', [
                'class' => 'ProductCarousel__Image',
            ]),
            'is_on_sale' => $product->isOnSale(),
            'discount_percentage' => sprintf('-%d%%', round($product->getDiscountPercentage())),
            'price' => wc_price($product->getPrice()),
            //'image' => $product->g(),
        ]);
    }
}