<?php
namespace Ep\Woo\Plugin\Shipping;

use Ep\Currency\Converter;
use Ak\Wp\Plugin\PluginInterface;

/**
 * Omogućava mijenjanje valute za naplatu dostave.
 *
 * @package Ep\Woo\Plugin\Shipping
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/woocommerce
 */
class Currency implements PluginInterface
{
    /** @var Converter */
    private $converter;

    /**
     * Currency constructor.
     *
     * @param Converter $converter
     */
    public function __construct(Converter $converter)
    {
        $this->converter = $converter;
    }

    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function attach(): void
    {
        add_filter('woocommerce_shipping_rate_cost', [$this, 'wc_shipping_rate_cost']);
        add_filter('woocommerce_shipping_rate_taxes', [$this, 'wc_shipping_rate_taxes']);
        add_filter('woocommerce_shipping_zone_shipping_methods', [$this, 'wc_shipping_zone_shipping_methods']);
    }

    /**
     * @param \WC_Shipping_Method[] $methods
     * @return \WC_Shipping_Method[]
     */
    public function wc_shipping_zone_shipping_methods($methods)
    {
        if (($currency = get_woocommerce_currency()) !== 'HRK') {
            foreach ($methods as $id => $method) {
                if ($method instanceof \WC_Shipping_Free_Shipping && $method->min_amount > 0) {
                    $method->min_amount = $this->converter->convert('HRK', $currency, $method->min_amount);
                }
            }
        }

        return $methods;
    }

    /**
     * @param $cost
     *
     * @return float
     */
    public function wc_shipping_rate_cost($cost)
    {
        if (($currency = get_woocommerce_currency()) !== 'HRK') {
            $cost = $this->converter->convert('HRK', $currency, floatval($cost));
        }

        return $cost;
    }

    /**
     * @param $taxes
     *
     * @return mixed
     */
    public function wc_shipping_rate_taxes($taxes)
    {
        if (($currency = get_woocommerce_currency()) !== 'HRK') {
            foreach ($taxes as $i => $tax) {
                $taxes[$i] = $this->converter->convert('HRK', $currency, floatval($tax));
            }
        }

        return $taxes;
    }
}